@@ -1,11 +1,11 @@
 #! perl
-# 	$rcs = ' $Id: testout.t,v 1.2 1997/09/22 10:13:37 ilya Exp ilya $ ' ;
+# 	$rcs = ' $Id: testout.t,v 1.2 1997/09/22 10:13:37 ilya Exp ilya $ ' ;	
 
 use Math::Pari qw(:DEFAULT pari_print :all);
 use vars qw($x $y $z $k $t $q $a $u $j $l $name $other $n);
 die "Need a path to a testout file" unless @ARGV;
 
-$file = CORE::shift;
+$file = shift;
 {
   open TO, "< $file" or die "open `$file': $!";
   local $/ = "\n? ";
@@ -13,26 +13,20 @@ $file = CORE::shift;
   close TO or die "close: $!";
 }
 
-$mess = CORE::shift @tests unless @tests and $tests[0] =~ /^\?/; # Messages
+$mess = shift @tests;		# Messages
 pop @tests;			# \q
-if ($tests = @tests) {
-  print "1..$tests\n";
-} else {
-  print "1..0 # skipped: no tests found in `$file'\n";
-}
+$tests = @tests;
 
-prec($3 || $1, 1) if ($mess || '') =~ /realprecision = (\d+) significant digits( \((\d+) digits displayed\))?/;
-
-#Math::Pari::dumpStack,allocatemem(8e6),Math::Pari::dumpStack if $file =~ /intnum/; # Due to our allocation policy we need more?
+print "1..$tests\n";
 
+prec($3 || $1, 1) if $mess =~ /realprecision = (\d+) significant digits( \((\d+) digits displayed\))?/;  
 
 $| = 1;
-@seen = qw(Euler Pi I getrand a x xx y z k t q u j l n v p e
-	   name other mhbi a2 a1 a0 b0 b1
-	   acurve bcurve ccurve cmcurve tcurve mcurve ma mpoints);
+@seen = qw(Pi I Euler getrand a x y z k t q u j l n v p name other mhbi a2 a1 a0 b0 b1
+	 acurve bcurve ccurve cmcurve tcurve mcurve ma mpoints);
 @seen{@seen}  = (' ', ' ', ' ', ' ', ('$') x 100);
 for (@seen) {
-  $$_ = PARI($_) unless $seen{$_} eq ' ';
+  $$_ = PARI($_);
 }
 $seen{'random'} = ' ';
 $DEFAULT = undef;
@@ -44,22 +38,17 @@ $DEFAULT = undef;
   )} = (1) x 10000;
 
 if ($file =~ /plot|graph|all/) {
-  if ($ENV{MP_NOGNUPLOT}) {
-    $skip_gnuplot = 1;
-  } else {
-    eval { link_gnuplot() };
-    if ($@ =~ m%^Can't locate Term/Gnuplot.pm in \@INC%) {
-      print STDERR "# Can't locate Term/Gnuplot.pm in \@INC, ignoring plotting\n";
-      @not_yet_defined{qw(
-	plotbox plotcolor plotcursor plotdraw ploth plothraw plotinit plotlines
-	plotmove plotpoints plotrline plotrmove plotrpoint psdraw psploth
-	psplothraw plotscale
-	plotkill
-      )} = (1) x 10000;
-      $skip_gnuplot = 1;
-    } elsif ($@) {
-      die $@;
-    }
+  eval { link_gnuplot() };
+  if ($@ =~ m%^Can't locate Term/Gnuplot.pm in \@INC%) {
+    print STDERR "# Can't locate Term/Gnuplot.pm in \@INC, ignoring plotting\n";
+    @not_yet_defined{qw(
+      plotbox plotcolor plotcursor plotdraw ploth plothraw plotinit plotlines 
+      plotmove plotpoints plotrline plotrmove plotrpoint psdraw psploth 
+      psplothraw
+      plotkill
+    )} = (1) x 10000;
+  } elsif ($@) {
+    die $@;
   }
 }
 
@@ -69,16 +58,15 @@ main_loop:
 for (@tests) {
   #print "Doing `$_'";
   1 while s/^\\\\.*\n//;	# Comment
-  $bad = /^\\/;			# \precision =
+  $bad = /^\\/;			# \precision = 
   $wasbadprint = /\b(plot)\b/;
   $wasprint = /\b((|p|tex)print(tex)?|plot)\b/;
   s/\s*\n\?\s*\Z// or die "Not terminated: `$_'\n";
   s/\A(\s*\?)+\s*//;
 #  s/[^\S\n]+$//gm;
   s/\A(.*)\s*; $ \s*(\w+)\s*\(/$1;$2(/mx; # Continuation lines of questions
-  # Special-case tests nfields-3 and intnum-23 with a wrapped question:
+  # Special-case test nfields-3 with a wrapped question:
   s/\A(p2=.*\d{10})\n(7\n)/$1$2/;
-  s/\n(?=\/\w)//;
   s/\A(.*)\s*$//m or die "No question: `$_'\n";
   $in = $1;
   1 while s/^\n//;		# Before warnings
@@ -87,7 +75,7 @@ for (@tests) {
     process_error($in,$_,$1);
     next;
   }
-  process_test($in, 'noans', []), next if /^$/; # Was a void
+  process_test($in, 'noans', []), next if /^$/; # Was a void 
 #  s/^%\d+\s*=\s*// or die "Malformed answer: $_" unless $bad or $wasprint;
   if ($_ eq '' or $wasprint) {	# Answer is multiline
 #    @ans = $_ eq '' ? () : ($_) ;
@@ -115,13 +103,13 @@ for (@tests) {
 }
 
 sub format_matrix {
-  my $in = CORE::shift;
+  my $in = shift;
   my @in = split /;/, $in;
   'PARImat_tr([[' . join('], [', @in) . ']])';
 }
 
 sub format_vvector {
-  my $in = CORE::shift;
+  my $in = shift;
   $in =~ s/~\s*$//;
   "PARIcol($in)";
 }
@@ -132,11 +120,6 @@ sub re_format {			# Convert PARI output to a regular expression
   $in =~ s/\\\]\\\n\\\n\\\[/\\s*;\\s*/g; # row separator
   $in =~ s/\\\n/\\s*/g;
   $in =~ s/\\[ \t]/,?\\s*/g;
-  # This breaks a lot of linear.t tests
-  ### Allow for rounding 3999999 <=> 40000000, but only after . or immediately before it
-  ##$in =~ s/([0-8])((?:\\\.)?)(9+(\\s\*9+)?)(?![+\d])/$n=$1+1;"($1$2$3|$n${2}0+)"/ge;
-  ##$in =~ s/([1-9])((?:\\\.)?)(0+(\\s\*0+)?)(?![+\d])/$n=$1-1;"($1$2$3|$n${2}9+)"/ge;
-  ##$in =~ s/(\d{8,})([1-8])(?![+\d()]|\\s\*)/$n=$2-1;$m=$2+1;$1."[$2$n$m]\\d*"/ge;
   $in
 }
 
@@ -162,7 +145,7 @@ sub mformat_transp {
   my @dummy = ('') x @{$arr[0]};
   for my $ind (0..$#{$arr[0]}) {
     for my $subarr (@arr) {
-      @$subarr > $ind or $subarr->[$ind] = '';
+      @$subarr > $ind or $subarr->[$ind] = ''; 
     }
     push @out, join ', ', map {$_->[$ind]} @arr;
   }
@@ -170,8 +153,8 @@ sub mformat_transp {
 }
 
 sub massage_floats {
-  my $in = CORE::shift;
-  $pre = CORE::shift || "16g";
+  my $in = shift;
+  $pre = shift || "16g";
   $in =~ s/(.\d*)\s+e/$1E/gi;	# 1.74 E-78
   $in =~ s/\b(\d+\.\d*(e[-+]?\d+)?|\d{10,})\b/sprintf "%.${pre}", $1/gei;
   $in;
@@ -202,9 +185,8 @@ sub process_test {
   $doprint = 1 if $noans eq 'print';
   my $was_prev = $prev;
   undef $prev;
-  $current_num++;
+  $c++;
   # First a trivial processing:
-  $in =~ s/^\s*gettime\s*;//;		# Starting lines of tests...
   $in =~ s/\b(\d+|[a-z]+\(\))\s*\\\s*(\d+(\^\d+)?)/ gdivent($1,$2)/g; # \
   $in =~ s/\b(\d+)\s*\\\/\s*(\d+)/ gdivround($1,$2)/g; # \/
   $in =~ s/\b(\w+)\s*!/ ifact($1)/g; # !
@@ -214,34 +196,18 @@ sub process_test {
   $in =~ s/(\w+)\s*\\(\w+(\s*\^\s*\w+)?)/gdivent($1,$2)/g; # random\10^8
   $in =~ s/%(?!\s*[\[_\w])/\$was_prev/g; # foo(%)
   $in =~ s/\b(for)\s*\(\s*(\w+)=/&$1($2,/g; # for(x=1,13,print(x))
-  $in =~ s/
-	    ^
-	    (
-	      \(
-		(?:
-		  [^(,)]+
-		  (?=
-		    [(,)]
-		  )
-		|
-		  \( [^()]* \)
-		)*		# One level of parenths supported
-	      \)
-	    )
-	    ' $
-	  /deriv$1/x; # ((x+y)^5)'
   if ($in =~ /^\\p\s*(\d+)/) {
     prec($1);
   } elsif ($in =~ /^\\ps\s*(\d+)/) {		# \\ for division unsupported
     sprec($1);
   } elsif ($in =~ /\\/) {		# \\ for division unsupported
-    $current_num--;
+    $c--;
     process_error($in, $out, '\\');
   } elsif ($in =~ /^(\w+)\s*\([^()]*\)\s*=/ and 0) { # XXXX Not implemented yet
-    $current_num--;
+    $c--;
     process_definition($1, $in);
   } elsif ($in =~ /[!\']/) {	# Factorial
-    print "# `$in'\nok $current_num # Skipping (ifact/deriv)\n";
+    print "# `$in'\nok $c # Skipping (ifact/deriv)\n";
   } else {
     # work with "^", need to treat differently inside o()
     $in =~ s/\^/^^^/g;
@@ -251,10 +217,10 @@ sub process_test {
     $in =~ s/\[([^\[\];]*)\]\s*~/format_vvector($1)/ge; # Vertical vector
  eval {
     1 while $in =~ s/
-	      \b (if|while|until) \(
+	      \b (if|while|until) \( 
 	      (
 		(?:
-		  [^(,)]+
+		  [^(,)]+ 
 		  (?=
 		    [(,)]
 		  )
@@ -262,10 +228,10 @@ sub process_test {
 		  \( [^()]* \)
 		)*		# One level of parenths supported
 	      )
-	      ,
+	      , 
 	      (
 		(?:
-		  [^(,)]+
+		  [^(,)]+ 
 		  (?=
 		    [(,)]
 		  )
@@ -274,10 +240,10 @@ sub process_test {
 		)*		# One level of parenths supported
 	      )
 	      (?:
-		,
+		, 
 		(
 		  (?:
-		    [^(,)]+
+		    [^(,)]+ 
 		    (?=
 		      [(,)]
 		    )
@@ -290,41 +256,40 @@ sub process_test {
 	    /process_cond($1, $2, $3, $4, $in)/xge; # if(a,b,c)
  };
     if ($in =~ /\[[^\]]*;/) {	# Matrix
-      print "# `$in'\nok $current_num # Skipping (matrix notation)\n";
+      print "# `$in'\nok $c # Skipping (matrix notation)\n";
       return;
     } elsif ($in =~ /Skip this `(.*)'/) {
-      print "# `$1'\nok $current_num # Skipping (runaway conversion)\n";
+      print "# `$1'\nok $c # Skipping (runaway conversion)\n";
       return;
     } elsif ($in =~ /&for\s*\([^\)]*$/) {	# Special case
-      print "# `$in'\nok $current_num # Skipping (runaway input line)\n";
+      print "# `$in'\nok $c # Skipping (runaway input line)\n";
       return;
     } elsif ($in =~ /(^|[\(=,])%/) {
-      print "# `$in'\nok $current_num # Skipping (history notation)\n";
-      return;
-    } elsif ($in =~ /\b(get(heap|stack)|Total time spent.*gettime)\b/) {
-      print "# `$in'\nok $current_num # Silently skipping: meaningless for Math::Pari\n";
+      print "# `$in'\nok $c # Skipping (history notation)\n";
       return;
     } elsif ($in =~ /
 		      (
-			\b
+			\b 
 			( if | goto | label | input | break
 			  # | while | until
-                          | gettime | default
+                          | gettime | default | sizebyte
+                          # XXXX These need to be done ASAP:
+			  | sumalt | prodinf 
                         )
-			\b
+			\b 
 		      |
-			(\w+) \s* \( \s* \w+ \s* \) =
+			(\w+) \s* \( \s* \w+ \s* \) = 
 		      |
 			\b install \s* \( \s* (\w+) \s* , [^()]* \)
 		      |
-			\b
+			\b 
 			(
 			  my _
 			)?
-			p? print \(
+			p? print \( 
 			( \[ | (1 ,)? PARImat )
 		      |	  # Too many parens: will not be wrapped in sub{...}
-		      	\b forprime .* \){5}
+		      	\b forprime .* \){4}
 		      )
 		    /x) {
       if (defined $3) {
@@ -344,244 +309,113 @@ sub process_test {
 	print "# Installed function `$4'.\n";
       }
       # It is not clear why changevar gives a different answer in GP
-      print "# `$in'\nok $current_num # Skipping (converting test for '$1' needs additional work)\n";
+      print "# `$in'\nok $c # Skipping (converting test for '$1' needs additional work)\n";
       return;
-    } elsif ($userfun
+    } elsif ($userfun 
 	     and $in =~ / \b ($userfun) \s* \( /x) {
-      print "# `$in'\nok $current_num # Skipping (user function)\n";
+      print "# `$in'\nok $c # Skipping (user function)\n";
       return;
     } elsif ($installed
 	     and $in =~ / \b ($installed) \s* \( /x) {
-      print "# `$in'\nok $current_num # Skipping (installed function)\n";
+      print "# `$in'\nok $c # Skipping (installed function)\n";
       return;
 #    } elsif ($in =~ / \b ( sizebyte ) \b /x
 #	     and $file !~ /will_fail/) {
 #      # XXXX Will result in a wrong answer, but we moved these tests to a different
-#      print "# `$in'\nok $current_num # Skipping (would fail, checked in different place)\n";
+#      print "# `$in'\nok $c # Skipping (would fail, checked in different place)\n";
 #      return;
-    } elsif ($in =~ /\b(nonesuch now|nfisincl)\b/) {
-      print "# `$in'\nok $current_num # Skipping (possibly FATAL $1)\n";
+    } elsif ($in =~ /\bget(heap|stack)\b/) { # Meaningless
+      print "# `$in'\nok $c # Skipping meaningless\n";
+      return;
+    } elsif ($in =~ /\b(nonesuch now)\b/) {
+      print "# `$in'\nok $c # Skipping (possibly FATAL $1)\n";
       return;
     }
     # Convert transposition
     $in =~ s/(\$?\w+(\([^()]*\))?|\[([^\[\]]+(?=[\[\]])|\[[^\[\]]*\])*\])~/mattranspose($1)/g;
     if ($in =~ /~/) {
-      print "# `$in'\nok $current_num # Skipping (transpose notation)\n";
+      print "# `$in'\nok $c # Skipping (transpose notation)\n";
       return;
     }
     if ($in =~ /^\s*alias\s*\(\s*(\w+)\s*,\s*(\w+)\s*\)$/) {
-      print "# Aliasing `$1' ==> `$2'\nok $current_num\n";
+      print "# Aliasing `$1' ==> `$2'\nok $c\n";
       *$1 = \&{$2};
       return;
     }
     if ($in !~ /\w\(/) { # No function calls
       # XXXX Primitive!
       # Constants: (.?) eats following + or - (2+3 ==> PARI(2)+PARI(3))
-      $in =~ s/(^|\G|\W)([-+]?)(\d+(\.\d*)?)(.?)/$1 $2 PARI($3) $5/g;
+      $in =~ s/(^|\G|\W)([-+]?\d+(\.\d*)?)(.?)/$1 PARI($2) $4/g;
       # Big integer constants:
       $in =~ s/\bPARI\((\d{10,})\)/PARI('$1')/g;
-    } elsif ($in =~ /\b(elllseries|binomial|mathilbert|intnum|intfuncinit|intfuncinit)\b/) { # high precision needed?
+    } elsif ($in =~ /\belllseries\b|\bbinomial\b|\*mathilbert\b/) { # high precision needed?
       # XXXX Primitive!
-      # Substitute constants where they are not arguments to functions,
-      # (except small half-integers, which should survive conversion)
-      $in =~ s/(^|\G|\W)([-+]?)(?!\d{0,6}\.5\b)(\d+\.\d*)/$1 $2 PARI('$3') /g;
-      # Bug in GP???  Too sensitive to precision of 4.5 in intfuncinit(t=[-oo, 4.5],[oo, 4.5], gamma(2+I*t)^3, 1);
-      $in =~ s/(^|\G|\W)([-+]?)(\d+\.\d*)/$1 $2 PARI('$3') /g
-	if $in =~ /intfuncinit/;
+      # Substitute constants where they are not arguments to functions
+      $in =~ s/(^|\G|\W)([-+]?\d+\.\d*)/$1 PARI('$2') /g;
       # Big integer constants:
       $in =~ s/\bPARI\((\d{10,})\)/PARI('$1')/g;
     } else {
       # Substitute big integer constants
       $in =~ s/(^|\G|\W)(\d{10,}(?!\.\d*))(.?)/$1 PARI('$2') $3/g;
-      # Substitute division
+      # Substitute division 
 	$in =~ s/(^|[\-\(,\[])(\d+)\s*\/\s*(\d+)(?=$|[\),\]])/$1 PARI($2)\/PARI($3) /g;
     }
-    %seen_now = ();
     # Substitute i= in loop commands
     if ($in !~ /\b(hermite|mathnf|until)\s*\(/) { # Special case, not loop-with-=
-      $in =~ s/([\(,])(\w+)=(?!=)/ $seen_now{$2} = '$'; "$1$2," /ge;
+      $in =~ s/([\(,]\w+)=(?!=)/$1,/g;
     }
     # Substitute print
     $in =~ s/\b(|p|tex)print(tex|)\(/ 'my_' . $1 . $2 . 'print(1,' /ge;
     $in =~ s/\b(|p|tex)print1\(/ 'my_' . $1 . 'print(0,'/ge;
     $in =~ s/\b(eval|shift|sort)\(/&$1\(/g; # eval($y)
-    # Special case -oo (with $oo=[PARI(1)] done earlier;
-    # Having $oo defined without external PARI tests conversions; but it'sn't overloaded
-    $in =~ s/-oo\b/- PARI(\$oo)/ if $seen_now{oo} or 1;
-    # Recognize variables and prepend $ in assignments
-    # s/\b(direuler\s*\(\s*\w+\s*),/$1=/;	# direuler
-    $in =~ s/\bX\b/PARIvar("X")/g if $in =~ /\bdireuler\b/;
-    $in =~ s/(^\s*|[;(]\s*)(?=(\w+)\s*=\s*)/$seen_now{$2} = '$'; $1 . '$'/ge; # Assignment
-    # Prepend $ to variables (not before '^' - inside of 'o(x^17)'):
-    $in =~ s/(^|[^\$])\b([a-zA-Z]\w*)\b(?!\s*[\(^])/
-      		($1 || '') . ($seen{$2} || $seen_now{$2} || '') . $2
-	/ge;
+    # Recognize variables
+    %seen_now = ();
+    $in =~ s/(^|[;(])(\w+)(\s*=\s*)/$seen_now{$2} = '$'; $1 . '$' . $2 . $3/ge; # Assignment
+    # Substitute variables (not before '^' - inside of 'o(x^17)'):
+    $in =~ s/(^|[^\$])\b([a-zA-Z]\w*)\b(?!\s*[(^])/($1 || '') . ($seen{$2} || $seen_now{$2} || '') . $2/ge;
     # Die if did not substitute variables:
     while ($in =~ /(^|[^\$])\b([a-zA-Z]\w*)\b(?!\s*[\{\(^])/g) {
-      print("# `$in'\nok $current_num # Skipping: variable `$2' was not set\n"), return
+      print("# `$in'\nok $c # Skipping ($2 was not set)\n"), return
 	unless $seen{$2} and $seen{$2} eq ' ' or $in =~ /\"/;
       # Let us hope that lines which contain '"' do not contain unset vars
     }
     # Simplify for the following conversion:
     $in =~ s/\brandom\(\)/random/g;
-    # Sub-ify sum,prod,intnum* sumnum etc, psploth, ploth etc
+    # Sub-ify sum,prod,intnum
     1 while
       $in =~ s/
 		(
-		  \b
-		  (?:		# For these, sub{}ify the fourth argument
-		    sum
+		  \b 
+		  (
+		    sum 
 		  |
-		    intnum(?!init\b)\w*
-		  |
-		    intfuncinit
-		  |
-		    int\w*inv
-		  |
-		    intcirc\w*
-		  |
-		    sumnum(?!init\b)\w*
-		  |
-		    forprime
-		  |
-		    psploth
-		  |
-		    ploth
+		    intnum
 		  |
 		    prod (?: euler )?
-		  |
-		    direuler
 		  ) \s*
-		  \(
+		  \( 
 		  (?:
 		     (?:
-		       [^(=,)\[\]]+ # One argument without (), []
-		       (?=
-		         [(=,)\[\]]
-		       )
-		     |		# One or two levels of parenths supported
-		       \( [^()]* \)
-		     |
-		       \( (?: [^()] | \( [^()]* \))* \)
-		     |		# Two levels of brackets supported
-		       \[ [^\[\]]* \]
-		     |
-		       \[ (?: [^\[\]] | \[ [^\[\]]* \] )* \]
-		     )*
-		  [,=]){3}		# $x,1,100
-	        |		# For these, sub{}ify the third argument
-		  \b
-		  (?:
-		    sumalt
-		  |
-		    prodinf
-		  )
-		  \(
-		  (?:
-		     (?:
-		       [^(=,)]+
+		       [^(=,)]+ 
 		       (?=
 		         [(=,)]
 		       )
 		     |
-		       \s*\w*\( [^()]+ \)\s*
-		     )*		# One level of func-call (PARI('.3')) supported
-		  [,=]){2}		# $x,1
-		)				# end group 1
+		       \( [^()]+ \)
+		     )		# One level of parenths supported
+		  [,=]){3}		# $x,1,100
+		)
 		(?!\s*sub\s*\{)	# Skip already converted...
-		(		# 2: This follows after a comma on toplevel
+		(		# This follows after a comma on toplevel
 		  (?:
-		    [^(,)\[\]]+
+		    [^(,)\[\]]+ 
 		    (?=
 		      [(,)\[\]]
 		    )
 		  |
 		    \(		# One level of parenths
 		    (?:
-		      [^()]+
-		      (?=
-			[()]
-		      )
-		    |
-		      \(	# Second level of parenths
-			(?:
-			  [^()]+
-			  (?=
-			    [()]
-			  )
-			  | \( [^()]* \) # Third level of parens
-			)*
-		      \)	# Second level of parenths ends
-		    )*
-		    \)
-		  |
-		    \[		# One level of brackets
-		    (?:
-		      [^\[\]]+
-		      (?=
-			[\[\]]
-		      )
-		    |
-		      \[ [^\[\]]+ \] # Second level of brackets
-		    )*
-		    \]
-		  )*		# Two levels of parenths supported
-		)				# end group 2
-		(?= [),] )
-	      /$1 sub{$2}/xg;
-    # Do the rest (do not take = after the variable name)
-    1 while
-      $in =~ s/
-		(
-		  \b
-		  (		# For these, sub{}ify the last argument
-		    solve
-		  |
-		    (?:
-		      post
-		    )?
-		    ploth (?! raw \b ) \w+
-		  |
-		    # sum \w*
-		    sum (?! alt | num (?:init)? \b) \w+
-		  |
-		    # prod \w*
-		    prodinf
-		  |
-		    v? vector v?
-		  |
-		    matrix
-		  |
-		    intgen
-		  #|
-		  #  intnum
-		  |
-		    intopen
-		  |
-		    for \w*
-		  )
-		  \(
-		  (?:
-		    [^()]+
-		    (?=
-		      [(,)]
-		    )
-		  |
-		    \( [^()]+ \)
-		  )+		# One level of parenths supported
-		  ,
-		)
-		(?!\s*sub\s*\{)	# Skip already converted...
-		(		# 3: This follows after a comma on toplevel
-		  (?:
-		    [^(,)\[\]]+
-		    (?=
-		      [()\[\]]
-		    )
-		  |
-		    \(		# One level of parenths
-		    (?:
-		      [^()]+
+		      [^()]+ 
 		      (?=
 			[()]
 		      )
@@ -592,7 +426,112 @@ sub process_test {
 		  |
 		    \[		# One level of brackets
 		    (?:
-		      [^\[\]]+
+		      [^\[\]]+ 
+		      (?=
+			[\[\]]
+		      )
+		    |
+		      \[ [^\[\]]+ \] # Second level of brackets
+		    )*
+		    \]
+		  )*		# Two levels of parenths supported
+		)
+		(?=
+                  \)
+                |
+		  , [^(,)]+ \)
+                )
+	      /$1 sub{$3}/xg;
+    # Sub-ify direuler
+    1 while
+      $in =~ s/
+		(
+		  \b 
+		  (
+		    direuler
+		  ) \s*
+		  \( 
+		  (?:
+		     (?:
+		       [^(=,)]+ 
+		       (?=
+		         [(=,)]
+		       )
+		     |
+		       \( [^()]+ \)
+		     )		# One level of parenths supported
+		  [,=]){3}		# $x,1,100
+		)
+		(?!\s*sub\s*\{)	# Skip already converted...
+                (.*)
+		(?=
+                  \)
+                )
+	      /$1 sub{$3}/xg;
+    # Do the rest
+    1 while
+      $in =~ s/
+		(
+		  \b 
+		  (
+		    solve 
+		  |
+		    (?:
+		      post (?! plothraw \b)
+		    )?
+		    ploth \w* 
+		  |
+		    # sum \w* 
+		    sum \w+
+		  |
+		    # prod \w* 
+		    prodinf
+		  |
+		    v? vector v? 
+		  |
+		    matrix 
+		  |
+		    intgen 
+		  #|
+		  #  intnum 
+		  |
+		    intopen 
+		  |
+		    for \w*
+		  )
+		  \( 
+		  (?:
+		    [^()]+ 
+		    (?=
+		      [(,)]
+		    )
+		  |
+		    \( [^()]+ \)
+		  )+		# One level of parenths supported
+		  ,
+		)
+		(?!\s*sub\s*\{)	# Skip already converted...
+		(		# This follows after a comma on toplevel
+		  (?:
+		    [^(,)\[\]]+ 
+		    (?=
+		      [()\[\]]
+		    )
+		  |
+		    \(		# One level of parenths
+		    (?:
+		      [^()]+ 
+		      (?=
+			[()]
+		      )
+		    |
+		      \( [^()]+ \) # Second level of parenths
+		    )*
+		    \)
+		  |
+		    \[		# One level of brackets
+		    (?:
+		      [^\[\]]+ 
 		      (?=
 			[\[\]]
 		      )
@@ -610,21 +549,12 @@ sub process_test {
     $in =~ s/([\w\)])\[/$1 -> [-1+/g;
     # Workaround for &eval test:
     $in =~ s/\$y=\$x;&eval\b(.*)/PARI('y=x');&eval$1;\$y=\$x/;
-    $in =~ s/\$yy=\$xx;&eval\b(.*)/PARI('yy=xx');&eval$1;\$yy=\$xx/;
-    # Workaround for hardly-useful support for &:
-    if ($in =~ s/([,\(]\s*)&(\$(\w+)\s*)(?=[,\)])/$1$2/) {
-      #$in = qq(\$$3 = PARI "'$3"; \$OUT = do { $in }; \$$3 = PARI "$3"; \$OUT)
-    }
     # Workaround for kill:
     $in =~ s/^kill\(\$(\w+)\);/kill('$1');\$$1=PARIvar '$1';/;
-    # Workaround for plothsizes:
-    $in =~ s/\bplothsizes\(/safe_sizes(/;
     print "# eval", ($noans ? "-$noans" : '') ,": $in\n";
     $printout = '';
-    my $have_floats = ($in =~ /\d+\.\d*|\d{10,}/
-		       or $in =~ /\b( ellinit|zeta|bin|comprealraw|frac|
-				      lseriesell|powrealraw|legendre|suminf|
-				      forstep )\b/x);
+    my $have_floats = ($in =~ /\d+\.\d*|\d{10,}/ 
+		       or $in =~ /\b(zeta|bin|comprealraw|frac|lseriesell|powrealraw|legendre|suminf|forstep)\b/);
     # Remove the value from texprint:
     # pop @$out if $in =~ /texprint/ and @$out == 2;
     $res = eval "$in";
@@ -643,13 +573,7 @@ sub process_test {
       }
       # New wrapping code gets in the way:
       $printout =~ s/\s+/ /g;
-      $rout =~ s/\t,/,/g;
       $rout =~ s/\s+/ /g;
-
-      $rout =~ s/,\s*/, /g;
-      $printout =~ s/,\s*/, /g;
-      $rout =~ s/\s*([-+])\s*/ $1 /g;
-      $printout =~ s/\s*([-+])\s*/ $1 /g;
     } else {
       # Special-case several tests in all.t
       if (($have_floats or $in =~ /^(sinh?|solve)\b/) and ref $res) {
@@ -657,17 +581,14 @@ sub process_test {
 	$rout = mformat @$out;
 	if (defined $rres and $rres !~ /\n/) {
 	  $rout =~ s/\]\s*\[/; /g;
-	  $rout =~ s/,\n/, \n/g; # Spaces were removed
+	  $rout =~ s/,\n/, \n/g; # Spaces were removed 
 	  $rout =~ s/\n//g;	# Long wrapped text
 	}
-	if ($rout =~ /\[.*[-+,;]\s/ or $rout =~ /\bQfb\b/) {
+	if ($rout =~ /\[.*[-+,;]\s/) {
 	  $rout =~ s/,*\s+/ /g;
 	  $rres =~ s/,*\s+/ /g if defined $res;
-	  $rres =~ s/,/ /g if defined $res;		# in 2.2.10 ", "
-	  $rout =~ s/;\s*/; /g;				# in 2.2.10 "; "
-	  $rres =~ s/;\s*/; /g if defined $res;		# in 2.2.10 "; "
 	}
-	if ($in =~ /\b(zeta|bin|comprealraw|frac|lseriesell|powrealraw|pollegendre|legendre|suminf|ellinit)\b/) {
+	if ($in =~ /\b(zeta|bin|comprealraw|frac|lseriesell|powrealraw|pollegendre|legendre|suminf)\b/) {
 	  $rres = massage_floats $rres, "14f";
 	  $rout = massage_floats $rout, "14f";
 	} else {
@@ -681,7 +602,7 @@ sub process_test {
 #	$rout = mformat @$out;
 #	if (defined $rres and $rres !~ /\n/) {
 #	  $rout =~ s/\]\s*\[/; /g;
-#	  $rout =~ s/,\n/, \n/g; # Spaces were removed
+#	  $rout =~ s/,\n/, \n/g; # Spaces were removed 
 #	  $rout =~ s/\n//g;	# Long wrapped text
 #	}
 #	if ($rout =~ /\[.*[-+,;]\s/) {
@@ -694,17 +615,11 @@ sub process_test {
     }
 
     if ($@) {
-      if ($@ =~ /^Undefined subroutine &main::(\w+)/
+      if ($@ =~ /^Undefined subroutine &main::(\w+)/ 
 	  and $not_yet_defined{$1}) {
-	print "# in='$in'\nok $current_num # Skipped: `$1' is known to be undefined\n";
-      } elsif ($@ =~ /high resolution graphics disabled/
-	       and not Math::Pari::have_graphics()) {
-	print "# in='$in'\nok $current_num # Skipped: graphic is disabled in this build\n";
-      } elsif ($@ =~ /gnuplot-like plotting environment not loaded yet/
-	       and $skip_gnuplot) {
-	print "# in='$in'\nok $current_num # Skipped: Term::Gnuplot is not loaded\n";
+	print "# in='$in'\nok $c # Skipped: `$1' is known to be undefined\n";
       } else {
-	print "not ok $current_num # in='$in', err='$@'\n";
+	print "not ok $c # in='$in', err='$@'\n";
       }
       return;
     }
@@ -712,29 +627,29 @@ sub process_test {
     if (defined $rres and defined $re_out) {
       $cmp = eval { $rres =~ /^$re_out$/ };
       if ($@ and $@ =~ /regexp too big/) {
-	print "ok $current_num # Skipped: $@\n";
+	print "ok $c # Skipped: $@\n";
 	@seen{keys %seen_now} = values %seen_now;
 	$prev = $res;
 	return;
       }
     }
-    if (not $noans and defined $re_out
+    if (not $noans and defined $re_out 
 	     and (not defined $rres or not $cmp)) {
       $out->[0] =~ s/\n/\t/g;	# @$out usually has 1 elt
-      print "not ok $current_num # in='$in'\n#    out='", $rres, "', type='", ref $res,
+      print "not ok $c # in='$in'\n#    out='", $rres, "', type='", ref $res,
       "'\n# pari==='", join("\t", @$out), "'\n# re_out='$re_out'\n";
     } elsif (not $noans and defined $re_out) {
-      print "ok $current_num\n";
+      print "ok $c\n";
       @seen{keys %seen_now} = values %seen_now;
       $prev = $res;
     } elsif (not $noans and (not defined $rres or $rres ne $rout)) {
-      print "not ok $current_num # in='$in'\n#    out='", $rres, "', type='", ref $res,
+      print "not ok $c # in='$in'\n#    out='", $rres, "', type='", ref $res,
       "'\n# expect='$rout'\n";
     } elsif ($doprint and $printout ne $rout) {
-      print "not ok $current_num # in='$in'\n# printout='", $printout,
+      print "not ok $c # in='$in'\n# printout='", $printout, 
       "'\n#   expect='$rout', type='", ref $res,"'\n";
     } else {
-      print "ok $current_num\n";
+      print "ok $c\n";
       @seen{keys %seen_now} = values %seen_now;
       $prev = $res;
     }
@@ -743,44 +658,44 @@ sub process_test {
 
 sub process_error {
   my ($in, $out, $error) = @_;
-  $current_num++;
-  print("# `$in'\nok $current_num # Skipping: test producing error unsupported yet ($error)\n");
+  $c++;
+  print("# `$in'\nok $c # Skipping error($error) test\n");
 }
 
 sub process_definition {
   my ($name, $def) = @_;
-  $current_num++;
+  $c++;
   eval "PARI('$def');  import Math::Pari $name;";
   if ($@) {
     chomp $@;
-    print("not ok $current_num # definition: `$in' error `$@'\n");
+    print("not ok $c # definition: `$in' error `$@'\n");
   } else {
-    print("# definition $current_num: `$in'\nok $current_num\n");
+    print("# definition $c: `$in'\nok $c\n");
   }
 }
 
 sub process_set {
   my ($in, $out) = @_;
   return process_test("setprecision($1)", 'noans', '') if $in =~ /^\\p\s*(\d+)$/;
-  $current_num++;
-  print("# `$in'\nok $current_num # Skipping setting test\n");
+  $c++;
+  print("# `$in'\nok $c # Skipping setting test\n");
 }
 
 sub process_print {
   my ($in, @out) = @_;
-  $current_num++;
-  print("# $current_num: `$in'\nok $current_num # Skipping plot() - can't test it yet\n");
+  $c++;
+  print("# $c: `$in'\nok $c # Skipping print\n");
 }
 
 sub process_multi {
   my ($in, $out) = @_;
   my @out = @$out;
-  $current_num++;
-  print("# `$in'\nok $current_num # Skipping multiline\n");
+  $c++;
+  print("# `$in'\nok $c # Skipping multiline\n");
 }
 
 sub my_print {
-  my $nl = CORE::shift;
+  my $nl = shift;
   @_ = map {(ref) ? (pari_print $_) : $_} @_;
   $printout .= join '', @_;
   $printout .= "\t" if $nl;
@@ -788,7 +703,7 @@ sub my_print {
 }
 
 sub my_pprint {
-  my $nl = CORE::shift;
+  my $nl = shift;
   @_ = map {(ref) ? (pari_pprint $_) : $_} @_;
   $printout .= join '', @_;
   $printout .= "\t" if $nl;
@@ -796,7 +711,7 @@ sub my_pprint {
 }
 
 sub my_texprint {
-  my $nl = CORE::shift;
+  my $nl = shift;
   @_ = map {(ref) ? (pari_texprint $_) : $_} @_;
   $printout .= join '', @_;
   $printout .= "\t" if $nl;
@@ -806,15 +721,10 @@ sub my_texprint {
 sub prec {
   setprecision($_[0]);
   print "# Setting precision to $_[0] digits.\n";
-  print "ok $current_num\n" unless $_[1];
+  print "ok $c\n" unless $_[1];
 }
 sub sprec {
   setseriesprecision($_[0]);
   print "# Setting series precision to $_[0] digits.\n";
-  print "ok $current_num\n";
+  print "ok $c\n";
 }
-
-# *Need* to convert to PARI, otherwise the arithmetic will propagate
-# the values to floats
-
-sub safe_sizes { eval {plothsizes()} or PARI [1000,1000,20,20,20,20]}
